// ----------------------------------------------------------------------------------------------------
// Callbacks requested for this integration service. Add "status" to add event status callback calls.
// ----------------------------------------------------------------------------------------------------
CALLBACKS = ANNOTATE_DELIVERY ? [ "deliveryStatus", "response" ] : [ "response" ];

/*
  Called when response object has been received from xMatters 
*/
function apia_response(apxml) {
    var apxmlAsObj = APXML.toResponseJs(apxml);
    apia_callback(apxmlAsObj);
}

function apia_callback(msg) {
  IALOG.debug("{0}Received message from xMatters:\nIncident: {1}\nEvent Id: {2}\nCallback Type: {3}", loggingHeader, msg.incident_id, msg.eventidentifier, msg.xmatters_callback_type);
    
  try {
  
    switch (msg.xmatters_callback_type) {

      case "response":
        try {
          handleResponse(msg);
        } catch (e) {
          handleException(msg, e);
        }
        break;
        
      case "status":  // not used in this integration, provided as a template
        handleEventStatus(msg);
        break;
      
      case "deliveryStatus":
        handleDeliveryStatus(msg);
        break;
    }
    
  } catch (e) {
    IALOG.error(loggingHeader + "apia_callback(" + msg.eventidentifier + ", " + msg.xmatters_callback_type + "): caught Exception - name: [" + e.name + "], message [" + e.message + "]: " + msg + "\n" + e);

    throw e;
  }    
}

/**
 * ---------------------------------------------------------------------------------------------------------------------
 * handleDeliveryStatus
 *
 * The main routine for handling delivery status messages from xMatters
 *
 * If an message is delivered successfully, a comment is added to the worklog of the Remedy Incident ticket quoting the user and device that was targeted
 * If an message delivery fails, a comment is added to the worklog of the Remedy Incident ticket quoting the user and device that was targeted
 * ---------------------------------------------------------------------------------------------------------------------
 */
function handleDeliveryStatus(msg) {

  if ( ANNOTATE_DELIVERY && msg.deliverystatus ) {    
    var cause = getProperty(msg, "cause", "");

    if (cause != null && cause.length > 0) {
      cause = "0x" + ("00000000" + cause).substr(-8);
    }
    
    switch (String(msg.deliverystatus).toLowerCase()) {
    
      case "delivered":
        logNotification("[Alarm " + getIncidentID(msg) + "][" + cause + "] Notification delivered successfully to " + msg.recipient + " | " + msg.device);
        break;
        
      case "failed": 
        logNotification("[Alarm " + getIncidentID(msg) + "][" + cause + "] Unable to deliver notification to " + msg.recipient + " | " + msg.device);
        break;
    }
  }
}

/**
 * ---------------------------------------------------------------------------------------------------------------------
 * handleResponse
 *
 * The main routine for handling user responses
 *
 * ---------------------------------------------------------------------------------------------------------------------
 */
function handleResponse(msg) {
  IALOG.debug("Enter - handleResponse");
  var incidentId = getIncidentID(msg);
  
  var cause = getProperty(msg, "cause", "");
  if (cause != null && cause.length > 0) {
    cause = "0x" + ("00000000" + cause).substr(-8);
  }

  var responder = msg.recipient;
  var device = msg.device;
  var annotation = "null".equals(msg.annotation) ? null : msg.annotation;

  IALOG.info("{0}: Received response '{1}' from {2}|{3} with comment '{4}'", incidentId, msg.response, responder, device, annotation);

  if (annotation == null || annotation == "") {
    annotation = "Received response '" + msg.response + "' by " + responder + " (" + device + ")";
  } else {
    annotation = annotation.replace('"',"") + " by " + responder + " (" + device + ")";
  }
  
  var responseType = "java.lang.String";
  var response = null;
  
  var notificationMessage = "[Alarm " + incidentId + "][" + cause + "] " + annotation;

  var execSpectrumCliResult = null;

  switch (String(msg.response).toLowerCase()) {

    case RESPONSE_ACTION_ACK:
    case RESPONSE_ACTION_ACKNOWLEDGE:
      var mhandle = getProperty(msg, "mhandle");
      var landscape = getProperty(msg, "landscape");

      var ack_params = " -a " + incidentId + " -l " + landscape;
      var update_params = " " + mhandle + " " + incidentId + " " + AlarmStatus + " \"" + annotation + "\"";

      execSpectrumCliResult = executeSpectrumCli(cmdPath, ack_cmd + ack_params, responseType);
      executeSpectrumCli(cmdPath, update_cmd + update_params, null);
      logNotification(notificationMessage);      
      break;
      
    case RESPONSE_ACTION_CLR:      
    case RESPONSE_ACTION_CLEAR:    
      var mhandle = getProperty(msg, "mhandle");
      var landscape = getProperty(msg, "landscape");
      
      var clear_params = " -a " + incidentId + " -l " + landscape;
      var update_params = " " + mhandle + " " + incidentId + " " + AlarmStatus + " \"" + annotation + "\"";
	
      execSpectrumCliResult = executeSpectrumCli(cmdPath, update_cmd + update_params, null);
      response = executeSpectrumCli(cmdPath, clear_cmd + clear_params, responseType);
      logNotification(notificationMessage);
      break;

    case RESPONSE_ACTION_IGN:
    case RESPONSE_ACTION_IGNORE:
      logNotification(notificationMessage);
      break;
      
    default:
      logNotification(notificationMessage);
      //IALOG.error("Unexpected response value: " + msg.response);
      break;
      
  }

  if (execSpectrumCliResult != null) {
    IALOG.debug(execSpectrumCliResult);
  }
  
  IALOG.debug("Exit - handleResponse");    
}

/**
 * ---------------------------------------------------------------------------------------------------------------------
 * handleException
 *
 * Notify a responder about an error during the processing of the response 
 *
 * @param msg message
 * @param e exception
 * ---------------------------------------------------------------------------------------------------------------------
 */
function handleException( msg, e ) {
  var incidentId = getIncidentID( msg );  
  var errMsg = "Caught exception processing [" + incidentId  + "|RECEIVED_RESPONSE|" + msg.response + "]: " + e;
  IALOG.error(errMsg);
  
  if (true) return; // This integration does not support error notifications
  
  var exceptionMessage = "CA Spectrum could not process the request and returned the following error message:\n\n " 
    + errMsg.replace(exceptionString, "") + ".\n\nTo resolve this issue, please contact your CA Spectrum Administrator."
 
  try {
    // Create APXML
    var apxml = ServiceAPI.createAPXML();

    apxml.setToken(ERROR_MESSAGE_BODY, exceptionMessage);
    apxml.setToken(ERROR_MESSAGE_SUBJECT, "Your response '" + msg.response + "' for CA Spectrum record " + incidentId + " was not processed");
    apxml.setToken(ERROR_MESSAGE_RESPONSE, msg.response);
    apxml.setToken("recipients", "[{\"targetName\":\"" + msg.recipient + "\"}]");  // expected by APXML.toJSON as an array of objects
    apxml.setToken("xmpriority", ERROR_MESSAGE_PRIORITY);
    apxml.setToken("agent_client_id", ServiceAPI.getConfiguration().getName());

    // Send event
    IALOG.debug("Sending error notification to [" + msg.recipient + "]");
    var response = XMRESTAPI.submitApxml(ERROR_FORM, apxml, null, null, null);
    
    if (response && response.status == XMRESTAPI.RESPONSE_SUCCESS) {
      IALOG.info("Error notification was sent to [" + msg.recipient + "]");
    } else if (response) {
      IALOG.error("Error notification was not sent to " + msg.recipient + ". xMatters returned response (" + response.status + ")");      
    } else {
      IALOG.error("Error notification was not sent to " + msg.recipient + " because of deduplication settings");      
    }
  } catch (e) {
    IALOG.error("Error notification was not sent to " + msg.recipient + " due to exception: " + e);      
  }  
}

/**
 * ---------------------------------------------------------------------------------------------------------------------
 * handleEventStatus 
 *
 * The template for handling event status messages from xMatters
 * ---------------------------------------------------------------------------------------------------------------------
 */
function handleEventStatus(msg)
{
  switch (msg.status) {
 
    case "active":     
      // "Event " + msg.eventidentifier + " successfully created in xMatters"
      break;
      
    case "terminated": 
      // "Event " + msg.eventidentifier + " terminated"
      break;
      
    case "terminated_external": // event terminated by user if msg.username != INITIATOR
      if (msg.username !== INITIATOR) {
        // "Event " + msg.eventidentifier + " manually terminated from within xMatters" /* + " by user [" + msg.username + "]" */ 
      } else {
        // Ignore the events created and terminated by this integration and not the actual user
      }
      break;
  }
}
